clearvars; close all; clc;

varNoise = .0005;
vLengthsc = 1.; 
hLengthsc = 1.; %.5 %3.

nTest = 100;
nTraining = 20;


xTest = linspace(-5,5,nTest)';
yTrue = sin(0.9*xTest);
%yTrue = .1*xTest.^2;

indexTraining = round(nTest*rand(nTraining,1));
xTraining = xTest(indexTraining,:);
yTraining = yTrue(indexTraining,:) + varNoise^.5*randn(nTraining,1);

K_SE = @(x1,x2,vLengthscale,hLengthscale) (vLengthscale^2*exp((-.5/(hLengthscale^2))*(repmat(transpose(x1),size(x2)) - repmat(x2,size(transpose(x1)))).^2));
    
K_trainTrain = K_SE(xTraining,xTraining,vLengthsc,hLengthsc) + varNoise*eye(nTraining);
K_trainTest = K_SE(xTraining,xTest,vLengthsc,hLengthsc);
K_testTrain = K_trainTest';
K_testTest = K_SE(xTest,xTest,vLengthsc,hLengthsc);

meanTest = K_trainTest*(K_trainTrain\yTraining); 
meanTest = meanTest';

covTest = K_testTest + varNoise - K_trainTest*(K_trainTrain\K_testTrain);
stdTest = diag(covTest).^.5;
stdTest = stdTest';

figure(1)
h = zeros(1, 3);
s = cell(1, 3);
s{1}='prediction (2 std)';
s{2}='true function';
s{3}='training data';

hold on;
h(1) = plot(xTest,meanTest,'k');
shadedErrorBar(xTest,meanTest,2*stdTest);
h(2) = plot(xTest,yTrue,'r');
h(3) = plot(xTraining,yTraining,'bx');
legend(h,s,'Orientation','horizontal','Location','SouthOutside');
legend('boxoff');
xlabel('x');
ylabel('y');
hold off;

figure;
SIGMA_prior = K_SE(xTest,xTest,vLengthsc,hLengthsc);
prior = chol(SIGMA_prior + 1e-6*eye(nTest),'lower')*randn(nTest,10);
plot(xTest,prior);
xlabel('x');
ylabel('y');
title('Samples of Squared Exponential Prior');

figure;
SIGMA_post = K_testTest - K_trainTest*(K_trainTrain\K_testTrain);
post = chol(SIGMA_post + 1e-6*eye(nTest),'lower')*randn(nTest,10) + meanTest';
plot(xTest,post);
hold on;
plot(xTraining,yTraining,'ro','LineWidth',4);
xlabel('x');
ylabel('y');
title('Samples of Squared Exponential Posterior');

disp('Hit any key to continue ...'); pause;

figure;
K_lin = @(x1,x2,sigma0) (sigma0^2+(repmat(transpose(x1),size(x2)) .* repmat(x2,size(transpose(x1)))) );
SIGMA_prior = K_lin(xTest,xTest,.5);
prior = chol(SIGMA_prior + 1e-6*eye(nTest),'lower')*randn(nTest,10);
plot(xTest,prior);
xlabel('x');
ylabel('y');
title('Samples of Linear Prior');

figure;
xTest = linspace(0,10,nTest)';
K_brown = @(x1,x2) (min(repmat(transpose(x1),size(x2)),repmat(x2,size(transpose(x1)))) );
SIGMA_prior = K_brown(xTest,xTest);
prior = chol(SIGMA_prior + 1e-2*eye(nTest),'lower')*randn(nTest,10);
plot(xTest,prior);
xlabel('x');
ylabel('y');
title('Samples of Brownian Motion Prior');




