clearvars; close all; clc;

nSamples = 750;

K=[.3 .05
   .05 .1];

K_trainTrain = .3;
K_trainTest = .05;
K_testTrain = K_trainTest';
K_testTest = .1;

mean_samples = [1 2]';
mean_train = 1;
mean_test = 2;
samplesTraining = 1.5;

nDimTest = 1;
dimTest = 2;
dimTraining = 1;

dimND = length(mean_samples);

% https://math.stackexchange.com/questions/2079137/generating-multivariate-normal-samples-why-cholesky
samples = chol(K + 1e-6*eye(dimND),'lower')*randn(dimND,nSamples) + repmat(mean_samples,1,nSamples);

xPDF = 0:.05:3.5;
yPDF = 0:.05:3;

[xPDFmesh, yPDFmesh] = meshgrid(xPDF,yPDF);
sizeMesh = size(xPDFmesh);
totalPoints = sizeMesh(1)*sizeMesh(2);
diffMesh = [reshape(xPDFmesh,1,totalPoints);reshape(yPDFmesh,1,totalPoints)] - repmat(mean_samples,1,totalPoints);

zPDFmesh = exp(diag(-.5*diffMesh'*(K\diffMesh))')/(det(2*pi*K)^.5);
zPDFmesh = reshape(zPDFmesh, sizeMesh);

figure(1)
plot(samples(1,:),samples(2,:),'kx');
xlabel('y_1');
ylabel('y_2');
title('2D Gaussian');
axis([0 3.5 0 3]);
disp('Hit any key to continue ...'); pause;
hold on;
contour(xPDFmesh,yPDFmesh,zPDFmesh,0:.1:1.5);

% mesh(xPDFmesh,yPDFmesh,zPDFmesh);
% axis([0 3.5 0 3 0 1]);
% xlabel('y_1');
% ylabel('y_2');
% zlabel('pdf');

disp('Hit any key to continue ...'); pause;
figure;
% https://stats.stackexchange.com/questions/30588/deriving-the-conditional-distributions-of-a-multivariate-normal-distribution
cond_nSamples = 100;
cond_mean = mean_test + K_trainTest*(K_trainTrain\(samplesTraining - mean_train));
cond_K = K_testTest - K_trainTest*(K_trainTrain\K_testTrain);

samplesTest = chol(cond_K + 1e-6*eye(nDimTest),'lower')*randn(nDimTest,cond_nSamples) + repmat(cond_mean,1,cond_nSamples);

cond_samples = zeros(dimND,cond_nSamples);
cond_samples(dimTest,:) = samplesTest;
cond_samples(dimTraining,:) = repmat(samplesTraining,1,cond_nSamples);

diffCond = yPDF - repmat(cond_mean,1,length(yPDF));
zPDFCond = exp(diag(-.5*diffCond'*(cond_K\diffCond))')/(det(2*pi*cond_K)^.5);

contour(xPDFmesh,yPDFmesh,zPDFmesh,0:.1:1.5);
hold on;
plot([samplesTraining samplesTraining],[-1 4],'r');
plot(zPDFCond+samplesTraining,yPDF,'b');
plot(cond_samples(1,:),cond_samples(2,:),'kx');
xlabel('y_1');
ylabel('y_2');
title('conditional 2D Gaussian');
axis([0 3.5 0 3]);

disp('Hit any key to continue ...'); pause;
figure(1)
plot(samples(1,1),samples(2,1),'xb','MarkerSize',10,'LineWidth',4);
plot(samples(1,2),samples(2,2),'xr','MarkerSize',10,'LineWidth',4);

figure(3)
dimSamples=repmat((1:1:dimND)',1,nSamples);
plot(dimSamples(:,1),samples(:,1),'b.-');
hold on;
plot(dimSamples(:,2),samples(:,2),'r.-');
xlabel('dim');
ylabel('y');
axis([.5 2.5 -0.5 3]);
xticks(dimSamples(:,1));
title('2D Gaussian');
disp('Hit any key to continue ...'); pause;
figure(3)
plot(dimSamples,samples,'k.-');
plot(dimSamples(:,1),samples(:,1),'b.-','LineWidth',4);
plot(dimSamples(:,2),samples(:,2),'r.-','LineWidth',4);

disp('Hit any key to continue ...'); pause;
figure;
dimSamples=repmat((1:1:dimND)',1,cond_nSamples);
plot(dimSamples,cond_samples,'.-');
hold on;
plot(1,samplesTraining,'ro','LineWidth',4);
xlabel('dim');
ylabel('y');
axis([.5 2.5 -0.5 3]);
xticks(dimSamples(:,1));
title('conditional 2D Gaussian');

disp('Hit any key to continue ...'); pause;
figure;
post_mean = [samplesTraining; cond_mean];
post_std = [0; cond_K^.5];
errorbar(dimSamples(:,1),post_mean,2*post_std);
hold on;
plot(1,samplesTraining,'ro','LineWidth',4);
xlabel('dim');
ylabel('y');
axis([.5 2.5 -0.5 3]);
xticks(dimSamples(:,1));
title('conditional 2D Gaussian (2 std)');

disp('Hit any key to continue ...'); pause;
figure;
nSamples = 100;
K = [.3  .05 0
     .05 .1 .04
      0  .04 .2];
  
mean_samples = [1 2 1]';

dimND = length(mean_samples);

samples = chol(K + 1e-6*eye(dimND),'lower')*randn(dimND,nSamples) + repmat(mean_samples,1,nSamples);
dimSamples=repmat((1:1:dimND)',1,nSamples);
plot(dimSamples,samples,'.-');
xlabel('dim');
ylabel('y');
title('3D Gaussian');
axis([.5 3.5 -0.5 3]);
xticks(dimSamples(:,1));

disp('Hit any key to continue ...'); pause;
figure;
nSamples = 10;
Kfct = @(x1,x2,sig) (exp((-.5/(sig^2))*(repmat(transpose(x1),size(x2)) - repmat(x2,size(transpose(x1)))).^2));
dimND = 10;
dimSamples = (1:dimND)';
lengthsc = 2;
K = Kfct(dimSamples,dimSamples,lengthsc);
mean_samples = zeros(dimND,1);
samples = chol(K + 1e-6*eye(dimND),'lower')*randn(dimND,nSamples) + repmat(mean_samples,1,nSamples);
dimSamples=repmat(dimSamples,1,nSamples);
plot(dimSamples,samples,'.-');
xlabel('dim');
ylabel('y');
title('10D Gaussian');
axis([.5 10.5 -2.5 2.5]);
xticks(dimSamples(:,1));

disp('Hit any key to continue ...'); pause;
figure;
nSamples = 10;
dimND = 10;
dimSamples = (1:dimND)';
dimTest = [2 3 4 6 7 8 10]';
nDimTest = 7;
dimTraining = [1 5 9]';
samplesTraining = [0 .5 -1]';
lengthsc = 2;
K_trainTrain = Kfct(dimTraining,dimTraining,lengthsc);
K_trainTest = Kfct(dimTraining,dimTest,lengthsc);
K_testTrain = K_trainTest';
K_testTest = Kfct(dimTest,dimTest,lengthsc);
mean_samples = zeros(dimND,1);

mean_train = mean_samples(dimTraining);
mean_test = mean_samples(dimTest);

cond_mean = mean_test + K_trainTest*(K_trainTrain\(samplesTraining - mean_train));
cond_K = K_testTest - K_trainTest*(K_trainTrain\K_testTrain);

samplesTest = chol(cond_K + 1e-6*eye(nDimTest),'lower')*randn(nDimTest,nSamples) + repmat(cond_mean,1,nSamples);

samples = zeros(dimND,nSamples);
samples(dimTest,:) = samplesTest;
samples(dimTraining,:) = repmat(samplesTraining,1,nSamples);
dimSamples=repmat(dimSamples,1,nSamples);
plot(dimSamples,samples,'.-');
hold on;
plot(dimTraining,samplesTraining,'ro','LineWidth',4);
xlabel('dim');
ylabel('y');
title('conditional 10D Gaussian');
axis([.5 10.5 -2.5 2.5]);
xticks(dimSamples(:,1));

disp('Hit any key to continue ...'); pause;
figure;
post_mean = zeros(dimND,1);
post_mean(dimTest,:) = cond_mean;
post_mean(dimTraining,:) = samplesTraining;
post_std = zeros(dimND,1);
post_std(dimTest,:) = diag(cond_K).^.5;
errorbar(dimSamples(:,1),post_mean,2*post_std);
hold on;
plot(dimTraining,samplesTraining,'ro','LineWidth',4);
xlabel('dim');
ylabel('y');
title('conditional 10D Gaussian (2 std)');
axis([.5 10.5 -2.5 2.5]);
xticks(dimSamples(:,1));
